@@ -1,9 +1,10 @@
 package version::vpp;
 use strict;
 
+use POSIX qw/locale_h/;
 use locale;
 use vars qw ($VERSION @ISA @REGEXS);
-$VERSION = "0.69_06";
+$VERSION = '0.76_01';
 $VERSION = eval $VERSION;
 
 push @REGEXS, qr/
@@ -22,18 +23,45 @@ use overload (
     'nomethod' => \&vnoop,
 );
 
+my $VERSION_MAX = 0x7FFFFFFF;
+
+eval "use warnings";
+if ($@) {
+    eval '
+	package warnings;
+	sub enabled {return $^W;}
+	1;
+    ';
+}
+
 sub new
 {
 	my ($class, $value) = @_;
 	my $self = bless ({}, ref ($class) || $class);
-	require POSIX;
-	my $currlocale = POSIX::setlocale(&POSIX::LC_ALL);
-	my $radix_comma = ( POSIX::localeconv()->{decimal_point} eq ',' );
+	
+	if ( ref($value) && eval('$value->isa("version")') ) {
+	    # Can copy the elements directly
+	    $self->{version} = [ @{$value->{version} } ];
+	    $self->{qv} = 1 if $value->{qv};
+	    $self->{alpha} = 1 if $value->{alpha};
+	    $self->{original} = ''.$value->{original};
+	    return $self;
+	}
+
+	my $currlocale = setlocale(LC_ALL);
+
+	# if the current locale uses commas for decimal points, we
+	# just replace commas with decimal places, rather than changing
+	# locales
+	if ( localeconv()->{decimal_point} eq ',' ) {
+	    $value =~ tr/,/./;
+	}
 
 	if ( not defined $value or $value =~ /^undef$/ ) {
 	    # RT #19517 - special case for undef comparison
 	    # or someone forgot to pass a value
 	    push @{$self->{version}}, 0;
+	    $self->{original} = "0";
 	    return ($self);
 	}
 
@@ -44,18 +72,11 @@ sub new
 	$value = _un_vstring($value);
 
 	# exponential notation
-	if ( $value =~ /\d+.?\d*e-?\d+/ ) {
+	if ( $value =~ /\d+.?\d*e[-+]?\d+/ ) {
 	    $value = sprintf("%.9f",$value);
-	    $value =~ s/(0+)$//;
+	    $value =~ s/(0+)$//; # trim trailing zeros
 	}
 	
-	# if the original locale used commas for decimal points, we
-	# just replace commas with decimal places, rather than changing
-	# locales
-	if ( $radix_comma ) {
-	    $value =~ tr/,/./;
-	}
-
 	# This is not very efficient, but it is morally equivalent
 	# to the XS code (as that is the reference implementation).
 	# See vutil/vutil.c for details
@@ -63,6 +84,7 @@ sub new
 	my $alpha = 0;
 	my $width = 3;
 	my $saw_period = 0;
+	my $vinf = 0;
 	my ($start, $last, $pos, $s);
 	$s = 0;
 
@@ -78,12 +100,11 @@ sub new
 	$start = $last = $pos = $s;
 		
 	# pre-scan the input string to check for decimals/underbars
-	while ( substr($value,$pos,1) =~ /[._\d]/ ) {
+	while ( substr($value,$pos,1) =~ /[._\d,]/ ) {
 	    if ( substr($value,$pos,1) eq '.' ) {
 		if ($alpha) {
-		    require Carp;
 		    Carp::croak("Invalid version format ".
-		    	"(underscores before decimal)");
+		      "(underscores before decimal)");
 		}
 		$saw_period++;
 		$last = $pos;
@@ -92,28 +113,37 @@ sub new
 		if ($alpha) {
 		    require Carp;
 		    Carp::croak("Invalid version format ".
-		    	"(multiple underscores)");
+			"(multiple underscores)");
 		}
 		$alpha = 1;
 		$width = $pos - $last - 1; # natural width of sub-version
 	    }
+	    elsif ( substr($value,$pos,1) eq ','
+		    and substr($value,$pos+1,1) =~ /[0-9]/ ) {
+		# looks like an unhandled locale
+		$saw_period++;
+		$last = $pos;
+	    }
 	    $pos++;
 	}
 
 	if ( $alpha && !$saw_period ) {
 	    require Carp;
-	    Carp::croak("Invalid version format (alpha without decimal)");
+	    Carp::croak("Invalid version format ".
+		"(alpha without decimal)");
 	}
 
 	if ( $alpha && $saw_period && $width == 0 ) {
 	    require Carp;
-	    Carp::croak("Invalid version format (misplaced _ in number)");
+	    Carp::croak("Invalid version format ".
+		"(misplaced _ in number)");
 	}
 
 	if ( $saw_period > 1 ) {
 	    $qv = 1; # force quoted version processing
 	}
 
+	$last = $pos;
 	$pos = $s;
 
 	if ( $qv ) {
@@ -153,9 +183,14 @@ sub new
 			    $orev = $rev;
 			    $rev += substr($value,$s,1) * $mult;
 			    $mult /= 10;
-			    if ( abs($orev) > abs($rev) ) {
-				require Carp;
-				Carp::croak("Integer overflow in version");
+			    if (   abs($orev) > abs($rev) 
+				|| abs($rev) > abs($VERSION_MAX) ) {
+				if ( warnings::enabled("overflow") ) {
+				    require Carp;
+				    Carp::carp("Integer overflow in version");
+				}
+				$s = $end - 1;
+				$rev = $VERSION_MAX;
 			    }
 			    $s++;
 			    if ( substr($value,$s,1) eq '_' ) {
@@ -168,9 +203,14 @@ sub new
 			    $orev = $rev;
 			    $rev += substr($value,$end,1) * $mult;
 			    $mult *= 10;
-			    if ( abs($orev) > abs($rev) ) {
-				require Carp;
-				Carp::croak("Integer overflow in version");
+			    if (   abs($orev) > abs($rev) 
+				|| abs($rev) > abs($VERSION_MAX) ) {
+				if ( warnings::enabled("overflow") ) {
+				    require Carp;
+				    Carp::carp("Integer overflow in version");
+				}
+				$end = $s - 1;
+				$rev = $VERSION_MAX;
 			    }
 			}
 		    }
@@ -186,6 +226,10 @@ sub new
 		    && substr($value,$pos+1,1) =~ /\d/ ) {
 		    $s = ++$pos;
 		}
+		elsif ( substr($value,$pos,1) eq ',' 
+		    && substr($value,$pos+1,1) =~ /\d/ ) {
+		    $s = ++$pos;
+		}
 		elsif ( substr($value,$pos,1) =~ /\d/ ) {
 		    $s = $pos;
 		}
@@ -218,13 +262,27 @@ sub new
 	}
 
 	if ( substr($value,$pos) ) { # any remaining text
-	    warn "Version string '$value' contains invalid data; ".
-	         "ignoring: '".substr($value,$pos)."'";
+	    if ( warnings::enabled("misc") ) {
+		require Carp;
+		Carp::carp("Version string '$value' contains invalid data; ".
+		     "ignoring: '".substr($value,$pos)."'");
+	    }
+	}
+
+	# cache the original value for use when stringification
+	if ( $vinf ) {
+	    $self->{vinf} = 1;
+	    $self->{original} = 'v.Inf';
+	}
+	else {
+	    $self->{original} = substr($value,0,$pos);
 	}
 
 	return ($self);
 }
 
+*parse = \&new;
+
 sub numify 
 {
     my ($self) = @_;
@@ -309,12 +367,11 @@ sub stringify
 	require Carp;
 	Carp::croak("Invalid version object");
     }
-    if ( exists $self->{qv} ) {
-	return $self->normal;
-    }
-    else {
-	return $self->numify;
-    }
+    return exists $self->{original} 
+    	? $self->{original} 
+	: exists $self->{qv} 
+	    ? $self->normal
+	    : $self->numify;
 }
 
 sub vcmp
@@ -402,13 +459,27 @@ sub is_alpha {
 }
 
 sub qv {
-    my ($value) = @_;
+    my $value = shift;
+    my $class = 'version';
+    if (@_) {
+	$class = ref($value) || $value;
+	$value = shift;
+    }
 
     $value = _un_vstring($value);
-    $value = 'v'.$value unless $value =~ /^v/;
-    return version->new($value); # always use base class
+    $value = 'v'.$value unless $value =~ /(^v|\d+\.\d+\.\d)/;
+    my $version = $class->new($value);
+    return $version;
 }
 
+*declare = \&qv;
+
+sub is_qv {
+    my ($self) = @_;
+    return (exists $self->{qv});
+}
+
+
 sub _verify {
     my ($self) = @_;
     if ( ref($self)
@@ -426,8 +497,8 @@ sub _un_vstring {
     my $value = shift;
     # may be a v-string
     if ( $] >= 5.006_000 && length($value) >= 3 && $value !~ /[._]/ ) {
-	my $tvalue = sprintf("%vd",$value);
-	if ( $tvalue =~ /^\d+\.\d+\.\d+$/ ) {
+	my $tvalue = sprintf("v%vd",$value);
+	if ( $tvalue =~ /^v\d+\.\d+\.\d+$/ ) {
 	    # must be a v-string
 	    $value = $tvalue;
 	}
@@ -435,60 +506,64 @@ sub _un_vstring {
     return $value;
 }
 
-# Thanks to Yitzchak Scott-Thoennes for this mode of operation
-{
-    local $^W;
-    *UNIVERSAL::VERSION = sub {
-	my ($obj, $req) = @_;
-	my $class = ref($obj) || $obj;
+sub _VERSION {
+    my ($obj, $req) = @_;
+    my $class = ref($obj) || $obj;
 
-	no strict 'refs';
-	eval "require $class" unless %{"$class\::"}; # already existing
-	return undef if $@ =~ /Can't locate/ and not defined $req;
-	
-	if ( not %{"$class\::"} and $] >= 5.008) { # file but no package
+    no strict 'refs';
+    eval "require $class" unless %{"$class\::"}; # already existing
+    return undef if $@ =~ /Can't locate/ and not defined $req;
+    
+    if ( not %{"$class\::"} and $] >= 5.008) { # file but no package
+	require Carp;
+	Carp::croak( "$class defines neither package nor VERSION"
+	    ."--version check failed");
+    }
+    
+    my $version = eval "\$$class\::VERSION";
+    if ( defined $version ) {
+	local $^W if $] <= 5.008;
+	$version = version::vpp->new($version);
+    }
+
+    if ( defined $req ) {
+	unless ( defined $version ) {
 	    require Carp;
-	    Carp::croak( "$class defines neither package nor VERSION"
-		."--version check failed");
-	}
-	
-	my $version = eval "\$$class\::VERSION";
-	if ( defined $version ) {
-	    local $^W if $] <= 5.008;
-	    $version = version::vpp->new($version);
-	}
+	    my $msg =  $] < 5.006 
+	    ? "$class version $req required--this is only version "
+	    : "$class does not define \$$class\::VERSION"
+	      ."--version check failed";
 
-	if ( defined $req ) {
-	    unless ( defined $version ) {
-		require Carp;
-		my $msg =  $] < 5.006 
-		? "$class version $req required--this is only version "
-		: "$class does not define \$$class\::VERSION"
-		  ."--version check failed";
-
-		if ( $ENV{VERSION_DEBUG} ) {
-		    Carp::confess($msg);
-		}
-		else {
-		    Carp::croak($msg);
-		}
+	    if ( $ENV{VERSION_DEBUG} ) {
+		Carp::confess($msg);
 	    }
+	    else {
+		Carp::croak($msg);
+	    }
+	}
 
-	    $req = version::vpp->new($req);
+	$req = version::vpp->new($req);
 
-	    if ( $req > $version ) {
-		require Carp;
+	if ( $req > $version ) {
+	    require Carp;
+	    if ( $req->is_qv ) {
 		Carp::croak( 
-		    sprintf ("%s version %s (%s) required--".
-		    "this is only version %s (%s)", $class, 
-		    $req->numify, $req->normal,
-		    $version->numify, $version->normal)
+		    sprintf ("%s version %s required--".
+			"this is only version %s", $class,
+			$req->normal, $version->normal)
+		);
+	    }
+	    else {
+		Carp::croak( 
+		    sprintf ("%s version %s required--".
+			"this is only version %s", $class,
+			$req->stringify, $version->stringify)
 		);
 	    }
 	}
+    }
 
-	return defined $version ? $version->numify : undef;
-    };
+    return defined $version ? $version->stringify : undef;
 }
 
 1; #this line is important and will help the module return a true value
